// dataMine Plugin for Vera
// (c) Chris Jackson


var seriesData = null;
var flotrGraph = null;
var flotrChan = [];
var flotrMax = 0;
var flotrMin = 0;
var	varUpdateLock = 0;
var gmtOffset = 0;
var jsonConfig = [];

var flotrOptions = {
			HtmlText: false,
			xaxis:{
				mode:"time"
			},
			selection: {
				mode: "x"
			},
			legend: {
				show: true,
				position: "ne"
			},
			mouse: {
				track: true,
				trackFormatter:function(obj) {
					var today = new Date(parseInt(obj.x)+gmtOffset);
					return today.toLocaleString() + ', ' + obj.y;
				},
				sensibility: 5,
				radius: 4
			},
			yaxis: {
				autoscale: true,
				autoscaleMargin: 0.01,
				showLabels: true
			},
			y2axis: {
				autoscale: true,
				autoscaleMargin: 0.01,
				showLabels: true
			}
		};

multiselectList = {
	msContainer: null,
	curVis: 0,
	selCnt: 0,
	elementList: null,

	initList : function(container) {
		this.selCnt = 0;
		this.msContainer = container;
		//document.getElementById(container).style.cssText = "position:absolute;float:left;width:400px;height:300px;display:inline-block;text-align:left;";
		document.getElementById(container).style.cssText = "position:absolute;float:left;height:280px;width:400px;overflow-y:scroll;display:inline-block;text-align:left;background-color:#FFFFcc;border:1px solid;";
		this.hide();
		this.elementList = new Array();
	},
	addElement: function(id, name) {
		this.elementList[id] = {'sel':0, 'axis':0};
		document.getElementById(this.msContainer).innerHTML += "<div id='li"+this.msContainer+id+"' style='display:block;width:380px;height:20px' "  +
					"onmouseover=this.style.backgroundColor=\"#e0e0e0\" onmouseout=this.style.backgroundColor=\"\" " +
					">" +
					"<div style=\"display:inline-block;width:20px;height:20px;vertical-align:middle;\" " +
					"onclick='multiselectList.goElement("+id+")' title='Graph \"" + name + "\"'>" +
					"<img id=\"go"+this.msContainer+id+"\" src='plugins/dataMine/bullet_go.png' width='16px' height='16px' >" +
					"</div>" +
					"<div id=\"lx"+this.msContainer+id+"\" style=\"display:inline-block;width:320px;height:20px;vertical-align:middle;\"" +
					"onclick='multiselectList.clickElement("+id+")' >" +
					name +
					"</div>" +
					"<div id=\"mu"+this.msContainer+id+"\" style=\"display:none;width:40px;height:20px;vertical-align:middle;\" >" +
					"<img id=\"mx"+this.msContainer+id+"\" src='plugins/dataMine/database_go.png' width='16px' height='16px' " +
					"onclick='multiselectList.goAll()' title='Graph all selected variables'>" +
					" " +
					"<img id=\"my"+this.msContainer+id+"\" src='plugins/dataMine/chart_yaxis.png' width='16px' height='16px' " +
					"onclick='multiselectList.toggleAxis("+id+")' title='Select plot axis'>" +
					"</div>" +
					"</div>";
	},
	clickElement: function(id) {
		if(this.elementList[id].sel == 0) {
			if(this.selCnt >= 4)
				return;
			$("mu"+this.msContainer+id).style.display="inline-block";
			this.elementList[id].sel = 1;
			this.selCnt++;
		}
		else {
			$("mu"+this.msContainer+id).style.display="none";
			this.elementList[id].sel = 0;
			this.selCnt--;
		}
	},
	goElement: function(id) {
		this.hide();
		flotrChan = []
		flotrChan[0] = {'id':0, 'axis':0};
		flotrChan[1] = {'id':0, 'axis':0};
		flotrChan[2] = {'id':0, 'axis':0};
		flotrChan[3] = {'id':0, 'axis':0};
		flotrChan[0].id   = id;
		flotrChan[0].axis = 0;
		doGraph(0,0);
	},
	toggleAxis: function(id) {
		if(this.elementList[id].axis == 0) {
			this.elementList[id].axis = 1;
			$("my"+this.msContainer+id).src   = 'plugins/dataMine/chart_yaxis2.png';
		}
		else {
			this.elementList[id].axis = 0;
			$("my"+this.msContainer+id).src   = 'plugins/dataMine/chart_yaxis.png';
		}
	},
	goAll: function() {
		this.hide();
		flotrChan = [];
		flotrChan[0] = {'id':0, 'axis':0};
		flotrChan[1] = {'id':0, 'axis':0};
		flotrChan[2] = {'id':0, 'axis':0};
		flotrChan[3] = {'id':0, 'axis':0};

		var out = 0;
		for(var s = 1; s < this.elementList.length; s++) {
			if(this.elementList[s].sel == 1) {
				flotrChan[out].id   = s;
				flotrChan[out].axis = this.elementList[s].axis;
				out++;
			}
		}
		doGraph(0,0);
	},
	hide: function() {
		$(this.msContainer).hide();
		this.curVis = 0;
	},
	show: function() {
		$(this.msContainer).show();
		this.curVis = 1;
	},
	toggleVis: function() {
		if(this.curVis == 0)
			this.show();
		else
			this.hide();
	},
	clearSelect: function() {
		for(var s = 0; s < this.elementList.length; s++) {
			$("mu"+this.msContainer+id).style.display="none";
			this.elementList[s].sel = 0;
		}
	},
	getSelect: function() {
		for(var s = 0; s < this.elementList.length; s++) {
			if(this.elementList[s].sel == 1)
				return s;
		}
	}
}

function doGraph(start, stop)
{
	$('flloading').show();

	// Time needs to be converted back to UTC for Vera!
	if(start != 0)
		start += gmtOffset/1000;
	if(stop != 0)
		stop += gmtOffset/1000;

	new Ajax.Request("/port_3480/data_request", {
		method: "get",
		parameters: {
			id: "lr_dmData",
			channel1: flotrChan[0].id,
			channel2: flotrChan[1].id,
			channel3: flotrChan[2].id,
			channel4: flotrChan[3].id,
			start: Math.floor(start),
			stop: Math.ceil(stop),
			format: "json"
		},
		onSuccess: function(transport) {
			// Stop the loading icon here in case there was an error and the json is screwed!
			$('flloading').hide();

			var json = transport.responseText.evalJSON();

			// Only display an updated graph if there are 2 points or more
			if(json.points > 1) {
				var binaryOffset = 0;
				var binaryAxis   = 0;

				var	cnt, tot;
				cnt = 0;
				tot = 0;
				for(var s = 0; s < json.series.length; s++) {
					if(flotrChan[s].axis == 0) {
						tot++;
						if(json.series[s].min == 0 & json.series[s].max == 1)
							cnt++;
					}
				}
				if(tot > 0 & tot == cnt) {
					binaryAxis = 1;
				}
				else {
					cnt = 0;
					tot = 0;
					for(var s = 0; s < json.series.length; s++) {
						if(flotrChan[s].axis == 1) {
							tot++;
							if(json.series[s].min == 0 & json.series[s].max == 1)
								cnt++;
						}
					}
					if(tot > 0 & tot == cnt)
						binaryAxis = 2;
				}

				if(binaryAxis == 0) {
					flotrOptions.yaxis.showLabels  = true;
					flotrOptions.y2axis.showLabels = true;
				}
				else if(binaryAxis == 1) {
					flotrOptions.yaxis.showLabels  = false;
					flotrOptions.y2axis.showLabels = true;
				}
				else if(binaryAxis == 2) {
					flotrOptions.yaxis.showLabels  = true;
					flotrOptions.y2axis.showLabels = false;
				}

				if(json.series) {
					// Update the time to compensate for timezone
					for(var s = 0; s < json.series.length; s++) {
						json.series[s].yaxis = flotrChan[s].axis+1;

						for(var i = 0; i < json.series[s].data.length; i++) {
							json.series[s].data[i][0] -= gmtOffset;
							if(binaryAxis == json.series[s].yaxis)
								json.series[s].data[i][1] += binaryOffset;
						}
						if(binaryAxis == json.series[s].yaxis)
							binaryOffset += 2;

						// If ticks are provided, add to the graph
						if(json.series[s].ticks) {
							flotrOptions.yaxis.ticks = new Array();
							for(var t = 0; t < json.series[s].ticks.length; t++) {
								var tmp = new Array();
								tmp.push(json.series[s].ticks[t][1],json.series[s].ticks[t][0]);
								if(json.series[s].yaxis == 1) {
									flotrOptions.yaxis.ticks.push(tmp);
								}
								else if(json.series[s].yaxis == 2) {
									flotrOptions.y2axis.ticks.push(tmp);
								}
							}
						}
						else {
							if(json.series[s].yaxis == 1)
								flotrOptions.yaxis.ticks  = null;
							else if(json.series[s].yaxis == 2)
								flotrOptions.y2axis.ticks = null;
						}

					}
					// Remember the series data so we can redraw if needed
					seriesData = json.series;

					// Draw the graph using the JSON data
					flotrGraph = Flotr.draw(document.getElementById('flcontainer'), json.series, flotrOptions);
				}
				if(json.min) {
					flotrMin = json.min - gmtOffset/1000;
					flotrMax = json.max - gmtOffset/1000;
				}
			}
		},
		onFailure: function() {
			$('flloading').hide();
		}
	});
}

function listGraph()
{
	new Ajax.Request("/port_3480/data_request", {
		method: "get",
		parameters: {
			id: "lr_dmList",
			format: "json"
		},
		onSuccess: function(transport) {
			var json = transport.responseText.evalJSON();

			if(json.list) {
				var selector = document.getElementById('fllist');
				for(var i = 0; i < json.list.length; i++) {
					multiselectList.addElement(json.list[i].id,json.list[i].name);
				}
			}
		},
		onFailure: function() {
		}
	});
}

function doGraphChan()
{
	multiselectList.hide();
	flotrChan = multiselectList.getSelect();
	doGraph(0,0);
}

function doGraphLeft()
{
	var scroll;

	scroll = (flotrMax - flotrMin) / 5;
	doGraph(flotrMin - scroll, flotrMax - scroll);
}

function doGraphRight()
{
	var scroll;

	scroll = (flotrMax - flotrMin) / 5;
	doGraph(flotrMin + scroll, flotrMax + scroll);
}

function doGraphZoomIn()
{
	var zoom;

	zoom = (flotrMax - flotrMin) / 5;
	doGraph(flotrMin + zoom, flotrMax - zoom);
}

function doGraphTime(days)
{
	var ts;

	ts = Math.round((new Date()).getTime() / 1000);
	doGraph(ts-(days*86400), ts);
}

function doGraphZoomOut()
{
	var zoom;

	zoom = (flotrMax - flotrMin) / 5;
	doGraph(flotrMin - zoom, flotrMax + zoom);
}

function doGraphAll()
{
	var ts;

	ts = Math.round((new Date()).getTime() / 1000);
	doGraph(0, ts);
}

function doShowLegend()
{
	if(flotrOptions.legend.show == false) {
		flotrOptions.legend.show = true;
		$("graf_legend").style.backgroundImage='';
	}
	else {
		flotrOptions.legend.show = false;
		$("graf_legend").style.backgroundImage='url(plugins/dataMine/bullet_delete.png)';
	}

	if(seriesData != null)
		flotrGraph = Flotr.draw(document.getElementById('flcontainer'), seriesData, flotrOptions);
}

function doShowPoints()
{
	if(flotrOptions.mouse.track == false) {
		flotrOptions.mouse.track = true;
		$("graf_points").style.backgroundImage='';
	}
	else {
		flotrOptions.mouse.track = false;
		$("graf_points").style.backgroundImage='url(plugins/dataMine/bullet_delete.png)';
	}

	if(seriesData != null)
		flotrGraph = Flotr.draw(document.getElementById('flcontainer'), seriesData, flotrOptions);
}

function addButton(id, img, func, title)
{
	return "<img id='"+id+"' src='"+img+"' width='20px' height='20px' onclick='"+func+"' onmouseover=this.style.backgroundColor=\"#e0e0e0\" onmouseout=this.style.backgroundColor=\"\" title=\""+title+"\">";
}

function addConfigButton(button, img, func, title)
{
	return "<img id='cfgbutton"+button+"' src='"+img+"' width='16px' height='16px' onclick='"+func+"' onmouseover=this.style.backgroundImage='url(plugins/dataMine/button_over.png)' onmouseout=this.style.backgroundImage='url("+img+")' title='"+title+"'>";
}

var msl;

// Entry function for history tab.
function showHistory(deviceId)
{
	// Flotr v2 includes
	if(navigator.appName == 'Microsoft Internet Explorer')
		include_remote_script("plugins/dataMine/flotr2.ie.min.js","");

	include_remote_script("plugins/dataMine/flotr2.min.js","");

	var d = new Date()
	gmtOffset = d.getTimezoneOffset()*60000;

	var htmlResult = "";

	htmlResult += "<table border='0' padding='0'>";
	htmlResult += "<tr style='padding: 1px 0;height:20px'>";

	htmlResult += "<td style=\"width:330px;\">";
	htmlResult += "<div style='display:inline-block;width:170px;height:20px;line-height:20px;vertical-align:middle;border:1px dotted;' ";
	htmlResult += "onclick='multiselectList.toggleVis()' ";
	htmlResult += "onmouseover=this.style.backgroundColor=\"#e0e0e0\" onmouseout=this.style.backgroundColor=\"\">";
	htmlResult += "<img src='plugins/dataMine/layers.png' width='164px' height='20px' title='Select variables to graph'>";
	htmlResult += "</div></td>"

	htmlResult += "<td style='border-left: 1px black dotted'>";
	htmlResult += addButton('graf_points',   'plugins/dataMine/comment.png',             'doShowPoints()',  'Enable/disable mouse tracking');
	htmlResult += addButton('graf_legend',   'plugins/dataMine/key.png',                 'doShowLegend()',  'Show/hide legend');
	htmlResult += "</td><td style='border-left: 1px black dotted'>";
	htmlResult += addButton('graf_day',      'plugins/dataMine/calendar_view_day.png',   'doGraphTime(1)',  'Show last day');
	htmlResult += addButton('graf_week',     'plugins/dataMine/calendar_view_week.png',  'doGraphTime(7)',  'Show last week');
	htmlResult += addButton('graf_month',    'plugins/dataMine/calendar_view_month.png', 'doGraphTime(28)', 'Show last month');
	htmlResult += "</td><td style='border-left: 1px black dotted'>";
	htmlResult += addButton('graf_graphall', 'plugins/dataMine/arrow_out.png',           'doGraphAll()',    'Show all data');
	htmlResult += addButton('graf_left',     'plugins/dataMine/arrow_left.png',          'doGraphLeft()',   'Scroll left');
	htmlResult += addButton('graf_right',    'plugins/dataMine/arrow_right.png',         'doGraphRight()',  'Scroll right');
	htmlResult += addButton('graf_zoomin',   'plugins/dataMine/zoom_in.png',             'doGraphZoomIn()', 'Zoom in');
	htmlResult += addButton('graf_zoomout',  'plugins/dataMine/zoom_out.png',            'doGraphZoomOut()','Zoom out');

	htmlResult += "</td></tr>";
	htmlResult +="</table>";
	htmlResult += "<div id=\"flcontainer\" style=\"position:absolute;float:left;width:570px;height:350px;\"></div>";
	htmlResult += "<div id=\"flloading\" style=\"position:absolute;float:left;width:580px;height:350px;display:inline-block;text-align: center;margin-top:150px;\"><img src='plugins/dataMine/loader.gif'></div>";
	htmlResult += "<div id=\"fllist\"></div>";

	set_panel_html(htmlResult);
	$('flloading').hide();

	multiselectList.initList("fllist");
	listGraph();

	// Hook into the 'flotr:select' event.
	$('flcontainer').observe('flotr:select', function(evt)
		{
			// Get the selected area coordinates passed as event memo.
			var area = evt.memo[0];

			// What happens: empty the container element, and draw a new
			// graph with bounded axis. The axis correspond to the selection
			doGraph(area.x1/1000, area.x2/1000);
		});

	if(flotrOptions.mouse.track == false) {
		$("graf_points").style.backgroundImage='url(plugins/dataMine/bullet_delete.png)';
	}

	if(flotrOptions.legend.show == false) {
		$("graf_legend").style.backgroundImage='url(plugins/dataMine/bullet_delete.png)';
	}

	if(seriesData != null)
		flotrGraph = Flotr.draw(document.getElementById('flcontainer'), seriesData, flotrOptions);

	return true;
}

function toggleDiv(dev, ser)
{
	var divid = "";

	if(jsonConfig.devices[dev].dmineExpanded == 0) {
		jsonConfig.devices[dev].dmineExpanded = 1;

		var button = 0;
		var dMineDiv = "";
		var deviceDiv = "";
		var serviceId = "";
		var serviceDiv = "";

		var deviceDiv = document.getElementById("dmine-device"+jsonConfig.devices[dev].id);

		var numStates= jsonConfig.devices[dev].states.length;
		for(var iState=0; iState<numStates; ++iState) {
			var serviceId  = "dmine-service" + jsonConfig.devices[dev].id + "::" + jsonConfig.devices[dev].states[iState].service;
			var serviceDiv = document.getElementById(serviceId);
			if(serviceDiv == null) {
				deviceDiv.innerHTML += "<div id=\"" + serviceId + "-h\" style=\"border-top:1px dotted black;\" onmousedown=\"toggleDiv(" + dev + ",'" + jsonConfig.devices[dev].states[iState].service+"')\"><b>Service: " + jsonConfig.devices[dev].states[iState].service + "</b></div>" +
									   "<div id=\"" + serviceId + "\" style=\"display:none;margin-left:20px;\"></div>";

				serviceDiv = document.getElementById(serviceId);
			}

			button++;
			serviceDiv.innerHTML += "<div id=\"dmine-var-"+jsonConfig.devices[dev].id+"::" + jsonConfig.devices[dev].states[iState].service+"::"+jsonConfig.devices[dev].states[iState].variable+"\""+
									" style=\"display:block;border-top:1px dotted grey;margin-right:10px;\">"+
									"<table><tr><td width=400px "+
									"onmousedown=\"toggleVarLogging("+jsonConfig.devices[dev].id+",'"+jsonConfig.devices[dev].states[iState].service+"','"+jsonConfig.devices[dev].states[iState].variable+"')\">"+
									jsonConfig.devices[dev].states[iState].variable + "</td>" +

									"<td id=\"dmine-but1--"+jsonConfig.devices[dev].id+"::" + jsonConfig.devices[dev].states[iState].service+"::"+jsonConfig.devices[dev].states[iState].variable+"\" style=\"display: none;\">"+
									addConfigButton(button, 'plugins/dataMine/lightbulb_off.png', 'varUpdate('+button+','+jsonConfig.devices[dev].id+',"'+jsonConfig.devices[dev].states[iState].service+'","'+jsonConfig.devices[dev].states[iState].variable+'")', "Variable not used in power calculation")+"</td>"+

									"<td id=\"dmine-but2--"+jsonConfig.devices[dev].id+"::" + jsonConfig.devices[dev].states[iState].service+"::"+jsonConfig.devices[dev].states[iState].variable+"\" style=\"display: none;\">"+
									addConfigButton(button, 'plugins/dataMine/lightbulb.png', 'varUpdate('+button+','+jsonConfig.devices[dev].id+',"'+jsonConfig.devices[dev].states[iState].service+'","'+jsonConfig.devices[dev].states[iState].variable+'")', "Variable is used in power calculation")+"</td>"+

									"<td id=\"dmine-but3-"+jsonConfig.devices[dev].id+"::" + jsonConfig.devices[dev].states[iState].service+"::"+jsonConfig.devices[dev].states[iState].variable+"\" style=\"display: none;\">"+
									addConfigButton(button, 'plugins/dataMine/time.png', 'varUpdate('+button+','+jsonConfig.devices[dev].id+',"'+jsonConfig.devices[dev].states[iState].service+'","'+jsonConfig.devices[dev].states[iState].variable+'")', "Regenerate history data")+"</td>"+

									"</tr></table></div>";
		}

		// Update the state of each variable
		new Ajax.Request("/port_3480/data_request", {
			method: "get",
			parameters: {
				id: "lr_dmList"
			},
			onSuccess: function(transport) {
				var json = transport.responseText.evalJSON();

				if(json.list)
					setVariableState(json.list);
			},
			onFailure: function() {
			}
		});

	}

	// Initially, dev was a pointer to the config array, now we need the actual device id
	dev = jsonConfig.devices[dev].id;

	if(ser == "")										// Device tab
		divid = "dmine-device"+dev;
	else												// Service tab
		divid = "dmine-service" + dev + "::" + ser;

	if(document.getElementById(divid).style.display == 'none') {
		document.getElementById(divid).style.display = 'block';
	}
	else {
		document.getElementById(divid).style.display = 'none';
	}
}







function setVariableState(list)
{
	for(var i = 0; i < list.length; i++) {
		var varId = "dmine-var-"+list[i].device+"::" +list[i].service+"::"+list[i].variable;
		var elId = document.getElementById(varId);
		if(elId != null) {
			if(list[i].logging == 1)
				elId.style.backgroundColor = "#00ff99";		// green
			else
				elId.style.backgroundColor = "#ffff99";		// yellow
		}

		varId = "dmine-but1-"+list[i].device+"::" +list[i].service+"::"+list[i].variable;
		elId = document.getElementById(varId);
		if(elId != null)
			elId.style.display = 'block';

		varId = "dmine-but2-"+list[i].device+"::" +list[i].service+"::"+list[i].variable;
		elId = document.getElementById(varId);
		if(elId != null)
			elId.style.display = 'block';

		varId = "dmine-but3-"+list[i].device+"::" +list[i].service+"::"+list[i].variable;
		elId = document.getElementById(varId);
		if(elId != null)
			elId.style.display = 'block';
	}
}

function toggleVarLogging(device, service, variable)
{
	new Ajax.Request("/port_3480/data_request", {
		method: "get",
		parameters: {
			id: "lr_dmCtrl",
			control: "list",
			device: device,
			service: service,
			variable: variable
		},
		onSuccess: function(transport) {
			var json = transport.responseText.evalJSON();
			if(json.list)
				setVariableState(json.list);
		},
		onFailure: function() {
		}
	});

 	return true;
}

function varUpdate(button, device, service, variable)
{
	// Only allow one transaction at a time!
	if(varUpdateLock == 1)
		return false;
	varUpdateLock = 1;

	var varId = "cfgbutton"+button;
	var elId = document.getElementById(varId);
	if(elId != null)
		elId.src = "plugins/dataMine/loader.gif";

	new Ajax.Request("/port_3480/data_request", {
		method: "get",
		parameters: {
			id: "lr_dmCtrl",
			control: "history",
			device: device,
			service: service,
			variable: variable
		},
		onSuccess: function(transport) {
			varUpdateLock = 0;
			if(elId != null)
				elId.src = "plugins/dataMine/time.png";

			var json = transport.responseText.evalJSON();
			if(json.list)
				setVariableState(json.list);
		},
		onFailure: function() {
			varUpdateLock = 0;
			if(elId != null)
				elId.src = "plugins/dataMine/time.png";
		}
	});

 	return true;
}

function showConfig(deviceId)
{
	var dMineDiv = "";

	// Populate the configuration list
	new Ajax.Request("/port_3480/data_request", {
		method: "get",
		parameters: {
			id: "user_data"
		},
		onSuccess: function(transport) {
			jsonConfig = transport.responseText.evalJSON();

			set_panel_html("<div style=\"width:570px;\" id=\"dMine-config\"></div>");

			dMineDiv = document.getElementById("dMine-config");

			// Loop through the JSON
			var numDevices = jsonConfig.devices.length;
			for(var iDevice=0; iDevice<numDevices; ++iDevice) {
				// This hasn't been expanded
				jsonConfig.devices[iDevice].dmineExpanded = 0;

				// Don't display invisible items
				if(jsonConfig.devices[iDevice].invisible == 1)
					continue;

				// New Device header
				dMineDiv.innerHTML += "<div id=\"dmine-device"+jsonConfig.devices[iDevice].id+"-b\" style=\"border-top:2px solid blue;margin:2px;\">" +
									  "<div id=\"dmine-device"+jsonConfig.devices[iDevice].id+"-h\" style=\"\" onmousedown=\"toggleDiv(" + iDevice + ",'')\"><b>Device: " + jsonConfig.devices[iDevice].name + "</b></div>" +
									  "<div id=\"dmine-device"+jsonConfig.devices[iDevice].id+"\" style=\"display:none;margin-left:20px;\"></div></div>";
			}

			// Update the state of each variable
			new Ajax.Request("/port_3480/data_request", {
				method: "get",
				parameters: {
					id: "lr_dmList"
				},
				onSuccess: function(transport) {
					var json = transport.responseText.evalJSON();

					if(json.list)
						setVariableState(json.list);
				},
				onFailure: function() {
				}
			});
		},
		onFailure: function() {
		}
	});

 	return true;
}

